@@ -70,7 +70,8 @@ typedef enum _CoglEGLWinsysFeature
   COGL_EGL_WINSYS_FEATURE_EGL_IMAGE_FROM_X11_PIXMAP     =1L<<1,
   COGL_EGL_WINSYS_FEATURE_EGL_IMAGE_FROM_WAYLAND_BUFFER =1L<<2,
   COGL_EGL_WINSYS_FEATURE_CREATE_CONTEXT                =1L<<3,
-  COGL_EGL_WINSYS_FEATURE_BUFFER_AGE                    =1L<<4
+  COGL_EGL_WINSYS_FEATURE_BUFFER_AGE                    =1L<<4,
+  COGL_EGL_WINSYS_FEATURE_FENCE_SYNC                    =1L<<5
 } CoglEGLWinsysFeature;
 
 typedef struct _CoglRendererEGL
@@ -82,6 +83,8 @@ typedef struct _CoglRendererEGL
   EGLint egl_version_major;
   EGLint egl_version_minor;
 
+  CoglClosure *resize_notify_idle;
+
   /* Data specific to the EGL platform */
   void *platform;
   /* vtable for platform specific parts */
@@ -115,8 +118,6 @@ typedef struct _CoglDisplayEGL
   EGLSurface current_draw_surface;
   EGLContext current_context;
 
-  CoglBool pending_resize_notify;
-
   /* Platform specific display data */
   void *platform;
 } CoglDisplayEGL;
@@ -158,6 +159,14 @@ _cogl_egl_destroy_image (CoglContext *ctx,
                          EGLImageKHR image);
 #endif
 
+#ifdef EGL_WL_bind_wayland_display
+CoglBool
+_cogl_egl_query_wayland_buffer (CoglContext *ctx,
+                                struct wl_buffer *buffer,
+                                int attribute,
+                                int *value);
+#endif
+
 CoglBool
 _cogl_winsys_egl_renderer_connect_common (CoglRenderer *renderer,
                                           CoglError **error);
