@@ -46,6 +46,11 @@
 #include "cogl-win32-renderer.h"
 #include "cogl-winsys-wgl-private.h"
 #include "cogl-error-private.h"
+#include "cogl-poll-private.h"
+
+/* This magic handle will cause g_poll to wakeup when there is a
+ * pending message */
+#define WIN32_MSG_HANDLE 19981206
 
 typedef struct _CoglRendererWgl
 {
@@ -161,6 +166,9 @@ _cogl_winsys_renderer_disconnect (CoglRenderer *renderer)
 {
   CoglRendererWgl *wgl_renderer = renderer->winsys;
 
+  if (renderer->win32_enable_event_retrieval)
+    _cogl_poll_renderer_remove_fd (renderer, WIN32_MSG_HANDLE);
+
   if (wgl_renderer->gl_module)
     g_module_close (wgl_renderer->gl_module);
 
@@ -227,16 +235,74 @@ win32_event_filter_cb (MSG *msg, void *data)
             }
         }
     }
+  else if (msg->message == WM_PAINT)
+    {
+      CoglOnscreen *onscreen =
+        find_onscreen_for_hwnd (context, msg->hwnd);
+      RECT rect;
+
+      if (onscreen && GetUpdateRect (msg->hwnd, &rect, FALSE))
+        {
+          CoglOnscreenDirtyInfo info;
+
+          /* Apparently this removes the dirty region from the window
+           * so that it won't be included in the next WM_PAINT
+           * message. This is also what SDL does to emit dirty
+           * events */
+          ValidateRect (msg->hwnd, &rect);
+
+          info.x = rect.left;
+          info.y = rect.top;
+          info.width = rect.right - rect.left;
+          info.height = rect.bottom - rect.top;
+
+          _cogl_onscreen_queue_dirty (onscreen, &info);
+        }
+    }
 
   return COGL_FILTER_CONTINUE;
 }
 
+static CoglBool
+check_messages (void *user_data)
+{
+  MSG msg;
+
+  return PeekMessageW (&msg, NULL, 0, 0, PM_NOREMOVE) ? TRUE : FALSE;
+}
+
+static void
+dispatch_messages (void *user_data)
+{
+  MSG msg;
+
+  while (PeekMessageW (&msg, NULL, 0, 0, PM_REMOVE))
+    /* This should cause the message to be sent to our window proc */
+    DispatchMessageW (&msg);
+}
+
 static CoglBool
 _cogl_winsys_renderer_connect (CoglRenderer *renderer,
                                CoglError **error)
 {
   renderer->winsys = g_slice_new0 (CoglRendererWgl);
 
+  if (renderer->win32_enable_event_retrieval)
+    {
+      /* We'll add a magic handle that will cause a GLib main loop to
+       * wake up when there are messages. This will only work if the
+       * application is using GLib but it shouldn't matter if it
+       * doesn't work in other cases because the application shouldn't
+       * be using the cogl_poll_* functions on non-Unix systems
+       * anyway */
+      _cogl_poll_renderer_add_fd (renderer,
+                                  WIN32_MSG_HANDLE,
+                                  COGL_POLL_FD_EVENT_IN,
+                                  check_messages,
+                                  dispatch_messages,
+                                  renderer);
+    }
+
   return TRUE;
 }
 
@@ -640,6 +706,10 @@ update_winsys_features (CoglContext *context, CoglError **error)
       g_strfreev (split_extensions);
     }
 
+  /* We'll manually handle queueing dirty events in response to
+   * WM_PAINT messages */
+  context->private_feature_flags |= COGL_PRIVATE_FEATURE_DIRTY_EVENTS;
+
   return TRUE;
 }
 
@@ -836,7 +906,9 @@ _cogl_winsys_onscreen_init (CoglOnscreen *onscreen,
 }
 
 static void
-_cogl_winsys_onscreen_swap_buffers (CoglOnscreen *onscreen)
+_cogl_winsys_onscreen_swap_buffers_with_damage (CoglOnscreen *onscreen,
+                                                const int *rectangles,
+                                                int n_rectangles)
 {
   CoglOnscreenWgl *wgl_onscreen = onscreen->winsys;
 
@@ -901,7 +973,8 @@ _cogl_winsys_wgl_get_vtable (void)
       vtable.onscreen_init = _cogl_winsys_onscreen_init;
       vtable.onscreen_deinit = _cogl_winsys_onscreen_deinit;
       vtable.onscreen_bind = _cogl_winsys_onscreen_bind;
-      vtable.onscreen_swap_buffers = _cogl_winsys_onscreen_swap_buffers;
+      vtable.onscreen_swap_buffers_with_damage =
+        _cogl_winsys_onscreen_swap_buffers_with_damage;
       vtable.onscreen_update_swap_throttled =
         _cogl_winsys_onscreen_update_swap_throttled;
       vtable.onscreen_set_visibility = _cogl_winsys_onscreen_set_visibility;
